//#define DS18B20_SENSOR
//#define MPU6050_SENSOR
//#define HC_SR04_SENSOR
//#define JOYSTICK
#define trigPin 8
#define echoPin 9

#define DIR 2
#include "thermometer_bmp.h"
#include "tank_bmp.h"
#include <OSD50HD.h>
#include <SdFat.h>
#include <SdFatUtil.h>
#include <SPI.h>
#include <avr/wdt.h>

#if defined(MPU6050_SENSOR)
#include <OneWire.h>
#include <DallasTemperature.h>
OneWire  ds(7);
#endif

#if defined(MPU6050_SENSOR)
#include <Wire.h>
#include <MPU6050.h>
MPU6050 mpu;
#endif

Sd2Card card;
SdVolume volume;
SdFile root;
SdFile file;
OSD50HD osd;
unsigned char buf[64];

File webFile;
SdFat SD;
int JX = 1250, JY = 500;
void setup() {
  //  delay(8000);
  Serial.begin (115200);
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
  pinMode(DIR, OUTPUT);
  digitalWrite(DIR, LOW);

  SdCardInit();
  //cross frame
  osd.SendLineHV(1, 650, 5, 1205, 7, 2, 0, 20); //
  delay(40);
  osd.SendLineHV(1, 1850, 5, 7, 1065, 2, 0, 21); //
  delay(40);
  osd.SendLineHV(1, 650, 1065, 1205, 7, 2, 0, 18); //
  delay(40);
  osd.SendLineHV(1, 650, 5, 7, 1065, 2, 0, 19); //
  delay(40);

  osd.SendLineHV(1, JX, 10, 1, 1065, 1, 0, 16); //cross X
  delay(40);
  osd.SendLineHV(1, 650, JY, 1205, 1, 1, 0, 17); //cross Y
  delay(40);

  delay(40);
  osd.SendGrid(1, 25, 76, 386, 256, 1, 40, 20, 1, 30); //Net settings
  delay(40);
  osd.SendLineHV(1, 20, 70, 376, 230, 2, 0, 15); //Net background
  delay(40);
  osd.SetPlot(1, 24, 85, 22, 20, 3, 1, 28); //Plot settings
  delay(40);
  osd.SendText(1, 2, 9, "Temperature plot");

  osd.SendGrid(1, 25, 320, 386, 501, 1, 40, 20, 1, 31); //Net settings
  delay(40);
  //SendLineHV(20,10,380,205,2,15); //Net background
  osd.SetPlot(1, 24, 330, 22, 20, 3, 1, 29); //Plot settings
  delay(40);
  osd.SendText(1, 2, 19, "Tank filling plot");

  //gyroscope
  osd.SendText(1, 23, 34, "Gyroscope:");
  osd.SendText(1, 23, 37, "Cross centre:");
  osd.SendCircle(1, 500, 770, 100, 100, 8, 1, 0, 8);

  osd.SendLineHV(1, 15, 580, 350, 460, 2, 0, 12); //background of tank
  delay(40);
  int dim = 2; //for 96x426 bitmap

  osd.SetBitmap(1, 35, 600, 120, 255, dim , 0, 1, 38);
  //osd.UpdateBitmap(0, 0, 5112, tank_bmp);
  BitmapFromFile(0, "tank.bmp", dim);


  osd.SetBitmap(1, 255, 585, 255, 255, dim , 0, 1, 39);
  BitmapFromFile(1, "therm.bmp", dim);
  //osd.UpdateBitmap(1, 0, 5112, thermometer_bmp);

#if defined(MPU6050_SENSOR)
  while (!mpu.begin(MPU6050_SCALE_2000DPS, MPU6050_RANGE_2G))
  {
    //    Serial.println("Could not find a valid MPU6050 sensor, check wiring!");
    delay(500);
  }
  delay(1000);
  while (!mpu.begin(MPU6050_SCALE_2000DPS, MPU6050_RANGE_2G))
  {
    delay(500);
  }
#endif

}
void loop() {
  // wdt_reset();
  AxisMeasure();
  TemperatureMeasure();
  DistanceMeasure();
  Joystick();
}
void SdCardInit(void)
{
  PgmPrintln("Card inserted. Initialisation...");
  if (!card.init(SPI_FULL_SPEED, 10))
  {
    PgmPrintln("card.init failed!");
    return;
  }
  if (!volume.init(&card)) PgmPrintln("vol.init failed!");
  PgmPrint("Volume is FAT");

  if (!root.openRoot(&volume)) PgmPrintln("openRoot failed");
  PgmPrintln("Files found in root:");
  root.ls(LS_DATE | LS_SIZE);

  // Recursive list of all directories
  PgmPrintln("Files found in all dirs:");
  root.ls(LS_R);

  PgmPrintln("Done");
}
void BitmapFromFile(unsigned char NrBmp, char* str, int dim)
{
  file.open(&root, str, O_READ);
  uint32_t size = file.fileSize();
  size -= 62; //wycięcie naglowka

  //Sending bitmap from file

  unsigned char b[10] = {HEADER, ID, UPGRADEBITMAP, 0, 0, 0, 0, 0, 0, 0};
  int cs = 0;
  b[3] = cs; //
  int adrress = 0;
  char*y = (char*)&adrress;
  b[4] = y[1];
  b[5] = y[0];
  y = (char*)&size; //sending lenght of the file
  b[6] = y[0];
  b[7] = y[1];
  b[8] = NrBmp; //bitmap number

  osd.SendFrame(b, 9);

  uint32_t FilePtr = 0;
  int BufSize = 512;
  uint16_t line = 1;
  line = (dim + 1) * 4;
  const uint16_t count[40] = {1280, 640, 426, 320, 256, 213, 182, 160, 142, 128, 116, 106, 98, 91, 85, 80, 75, 71, 67, 64, 60,
                              58, 55, 53, 51, 49, 47, 45, 44, 42, 41, 40, 38, 37, 36, 35, 34, 33, 32, 32
                             };

  file.read(buf, 62); //Header reading
  for (int i = 0; i < count[dim]; i++) //426
  {
    file.read(buf, line);
    for (int t = 0; t < line; t++)
      Serial.write(buf[line - t - 1]);
  }
  file.close();
}
void TemperatureMeasure(void)
{
  static byte i, conversion = 0;
  static byte present = 0;
  static  byte data[12];
  static byte addr[8];
  static unsigned char tab[20];
  static uint32_t tit = 0, TWT = 0;
  if (millis() - tit > TWT)
  {
    if (conversion == 0)
    {
#if defined(DS18B20_SENSOR)
      ds.search(addr);
      ds.reset_search();
      if ( OneWire::crc8( addr, 7) != addr[7]) {
        // Serial.print("CRC is not valid!\n");
        return;
      }

      ds.reset();
      ds.select(addr);
      ds.write(0x44, 1);        // start conversion
#endif
      conversion = 1;
      TWT = 800;
      tit = millis();
    }
    else if (conversion == 1)
    {

#if defined(DS18B20_SENSOR)
      present = ds.reset();
      ds.select(addr);
      ds.write(0xBE);         // Read Scratchpad

      for ( i = 0; i < 9; i++) {           // we need 9 bytes
        data[i] = ds.read();
      }
      //conversion

      int HighByte, LowByte;
      LowByte = data[0];
      HighByte = data[1];
      float t = (HighByte << 8) + LowByte;
      t /= 16;
#else
      static int demo_temp = 0;
      float t = 6 * (4.3 + sin((float)demo_temp / 30));
      demo_temp++;
#endif
      char tmp[10];
      dtostrf(t, 1, 2, tmp);
      osd.SendText(1, 13, 22, tmp);
      int tt = - t * 4;

      osd.SendLineHV(1, 278, 833 + tt, 10, 135 - tt, 1, 0, 14); //temperature bar
      tt *= 4;
      tt += 550;
      if (tt < 0)
        tt = 0;
      if (tt > 253)
        tt = 253;
      osd.UpdatePlot(1, 0, tt);
      conversion = 0;
      TWT = 10;
      tit = millis();
    }
  }
}

void AxisMeasure(void)
{

#if defined(MPU6050_SENSOR)
  Vector rawAccel = mpu.readRawAccel();
  Vector normAccel = mpu.readNormalizeAccel();
  Vector rawGyro = mpu.readRawGyro();
  Vector normGyro = mpu.readNormalizeGyro();
#endif


  char tmp[10];
  float off = 1.4;
  float X1, Y1, X2, Y2, iX = 500, iY = 770;
  static float angle = PI / 2;
  //double arg=TrackBar4->Position;
  static double arg = 0;
#if defined(MPU6050_SENSOR)
  angle += (normGyro.ZAxis + off) / 10 * PI / 180;
  float angleAcc = -0.02 - atan(normAccel.YAxis / normAccel.XAxis) + PI / 2;
#else
  static float demo_angle = 1, angle_step = 0.01;
  demo_angle += angle_step;
  if (demo_angle > 2)
    angle_step = -angle_step;
  if (demo_angle < 1)
    angle_step = -angle_step;
  float angleAcc =  demo_angle;
  angle = angleAcc;
#endif
  float angleGyr = angle;
  arg = 0.9 * angleGyr + 0.1 * angleAcc;
  angle = arg;

  float wyswkat = (angle - PI / 2) * 180 / PI;
  dtostrf(wyswkat, 1, 2, tmp);
  osd.SendText(1, 23, 35, "angle =         ");
  osd.SendText(1, 31, 35, tmp);
  //dtostrf(floatVar, minStringWidthIncDecimalPoint, numVarsAfterDecimal, charBuf);
  dtostrf(JY - 10, 1, 2, tmp);
  osd.SendText(1, 23, 38, "JY =         ");
  osd.SendText(1, 28, 38, tmp);

  dtostrf(JX - 660, 1, 2, tmp);
  osd.SendText(1, 23, 39, "JX =        ");
  osd.SendText(1, 28, 39, tmp);

  float lenght = 93;
  X1 = lenght * sin(arg);
  Y1 = lenght * cos(arg);

  //int iX1=X1;
  //int iY1=Y1;

  int x1 = iX + X1;
  int x2 = iX - X1;
  if (x1 == x2)
    x2++;

  osd.SendLine(1, x1, Y1 + iY, x2, iY - Y1, 25, 2, 1, 0);


  static float wsp = 1;
  wsp = (wsp * 0.9) + (0.1 * angle / (PI / 2));
  //ograniczenie zeby kolko nie wylatywalo
  if (wyswkat > 2.8)
  {
    if (wsp > 1.028)
      wsp = 1.028;
  }
  if (wyswkat < -2.8)
  {
    if (wsp < 0.972)
      wsp = 0.972;
  }
  float dx = 30 * X1, dy = 30 * Y1;

  int cx = (iX - dx) * (1 - wsp) + iX * (wsp);
  int cy = (iY - dy) * (1 - wsp) + iY * (wsp);
  osd.SendCircle(1, cx, cy , 8, 8, 1, 1, 0,  9);
}
void DistanceMeasure(void)
{
  static long duration, distance;
  uint16_t slit = 0;
#if defined(HC_SR04_SENSOR)
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  duration = pulseIn(echoPin, HIGH);
  duration /= (2 * 29.1);
  distance = duration;
#else
  static int demo_distance = 0;
  distance = 10 * (1.5 + sin((float)demo_distance / 50));
  demo_distance++;
#endif

  uint16_t lit = 25 - distance;
  if (lit < 0)
    lit = 0;
  static float sl = 0;
  float l = lit;
  sl = (0.95 * sl + 0.05 * lit);
  slit = sl * 15;

  if (slit > 325)
    slit = 325;
  char temp[10];
  temp[0] = 0;
  dtostrf(slit, 1, 2, temp);
  strcat(temp, "L");
  osd.SendText(1, 3, 22, temp);
  int d = 350 - slit;
  osd.SendLineHV(1, 47, 620 + d, 130, 350 - d, 1, 0, 13); //13 tank bar
  d = float(d) / 1.28;
  d -= 25;
  if (d < 0)
    d = 0;
  if (d > 253)
    d = 253;
  osd.UpdatePlot(1, 1, d);
}
void Joystick(void)
{
  //joystick
  char tmp[10];
#if defined(HC_SR04_SENSOR)
  int dx = analogRead(A0) - 512; // must be calibrated
  int dy = analogRead(A1) - 512; // must be calibrated
  JX += (dx / 10);
  JY -= (dy / 10);
  if (JX < 660)
    JX = 660;
  if (JX > 1848)
    JX = 1848;
  if (JY < 10)
    JY = 10;
  if (JY > 1065)
    JY = 1065;
#else
  static int demo_step_x = 40, demo_step_y = 40;
  int dx = demo_step_x;
  int dy = demo_step_y;
  /*
    if (JX <= 660)
    demo_step_x=-demo_step_x;
    if (JX >= 1848)
    demo_step_x=-demo_step_x;
    if (JY <= 10)
    demo_step_y=-demo_step_y;
    if (JY >= 1065)
    demo_step_y=-demo_step_y;
  */
  JX += (dx / 10);
  JY -= (dy / 10);
  if (JX < 660)
  {
    JX = 660;
    demo_step_x = -demo_step_x;
  }
  if (JX > 1848)
  {
    JX = 1848;
    demo_step_x = -demo_step_x;
  }
  if (JY < 10)
  {
    JY = 10;
    demo_step_y = -demo_step_y;
  }
  if (JY > 1065)
  {
    JY = 1065;
    demo_step_y = -demo_step_y;
  }
#endif

  osd.SendLineHV(1, JX, 10, 1, 1065, 1, 1, 16); //cross X
  osd.SendLineHV(1, 650, JY, 1200, 1, 1, 1, 17); //cross Y
}
